using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Web.Mvc;

namespace samples {
   
   public static class ViewDataExtensions {

      public static void SetSelectList<TModel, TProperty>(
         this ViewDataDictionary viewData,
         TModel model,
         Expression<Func<TModel, TProperty>> propertySelector,
         IEnumerable<SelectListItem> list) {

         viewData[ExpressionHelper.GetExpressionText(propertySelector)] = list;
      }

      public static void SetSelectList<TModel, TProperty>(
         this ViewDataDictionary viewData,
         TModel model,
         Expression<Func<TModel, TProperty>> propertySelector,
         IDictionary<TProperty, string> data) {

         SetSelectList(viewData, model, propertySelector, new SelectList(data, "Key", "Value", (model != null) ? propertySelector.Compile()(model) : default(TProperty)));
      }

      public static void SetSelectList<TModel, TProperty>(
         this ViewDataDictionary viewData,
         TModel model,
         Expression<Func<TModel, Nullable<TProperty>>> propertySelector,
         IDictionary<TProperty, string> data) where TProperty : struct {

         SetSelectList(viewData, model, propertySelector, new SelectList(data, "Key", "Value", (model != null) ? propertySelector.Compile()(model) : default(TProperty?)));
      }

      public static void SetSelectList<TModel, TProperty>(
         this ViewDataDictionary viewData,
         TModel model,
         Expression<Func<TModel, TProperty[]>> propertySelector,
         IDictionary<TProperty, string> data) {
         
         SetSelectList(viewData, model, propertySelector, new MultiSelectList(data, "Key", "Value", (model != null) ? propertySelector.Compile()(model) : default(TProperty[])));
      }
   }
}