﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using maxtoroq.web.mvc.html;

namespace samples.helpers {

   public class HelpersController : Controller {

      public ActionResult Index() {
         return View();
      }

      public ActionResult Pager(
         [Bind(Prefix = "s", Include = "MaxPagesToDisplay,PreviousText,NextText,FirstLastSeparator,AlwaysDisplayPreviousNext,AlwaysDisplayFirstLast,DisplaySummary,SummaryFormatString")]
         PagerSettings settings
         , int start = 0) {

         if (settings == null)
            settings = new PagerSettings();

         var list = from t in typeof(Controller).Assembly.GetTypes()
                    where t.IsPublic
                    orderby t.FullName
                    select t;

         const int pageSize = 10;

         this.ViewData.Model = list.Skip(start).Take(pageSize).ToArray();
         this.ViewBag.PageSize = 10;
         this.ViewBag.Count = list.Count();
         this.ViewBag.start = start;
         this.ViewBag.settings = settings;

         return View();
      }

      public ActionResult CheckBoxList(DayOfWeek[] day) {

         this.ViewData.Model = day;
         this.ViewBag.day = EnumList.CreateMulti(typeof(DayOfWeek));

         return View();
      }

      public ActionResult CheckBoxListGrouped(string[] countries) {

         var countriesDic = new Dictionary<string, string>();

         foreach (CultureInfo culture in CultureInfo.GetCultures(CultureTypes.SpecificCultures)) {

            var region = new RegionInfo(culture.Name);

            if (!countriesDic.ContainsKey(region.TwoLetterISORegionName))
               countriesDic.Add(region.TwoLetterISORegionName, region.EnglishName);
         }

         this.ViewData.Model = countries;
         this.ViewBag.countries =
            from pair in countriesDic
            orderby pair.Value
            select new GroupedSelectListItem {
               GroupName = pair.Value.Substring(0, 1),
               Text = pair.Value,
               Value = pair.Key,
               Selected = countries != null
                  && countries.Contains(pair.Key)
            };

         return View();
      }

      public ActionResult CheckBoxListFor(DaysModel model) {

         this.ViewData.Model = model;
         this.ViewData.SetSelectList(model, m => m.Days, EnumList.CreateMulti(typeof(DayOfWeek)));

         return View();
      }

      public ActionResult RadioButtonList(DayOfWeek? day) {

         this.ViewData.Model = day;
         this.ViewBag.day = EnumList.Create(typeof(DayOfWeek));

         return View();
      }

      public ActionResult RadioButtonListGrouped(string country) {

         var countriesDic = new Dictionary<string, string>();

         foreach (CultureInfo culture in CultureInfo.GetCultures(CultureTypes.SpecificCultures)) {

            var region = new RegionInfo(culture.Name);

            if (!countriesDic.ContainsKey(region.TwoLetterISORegionName))
               countriesDic.Add(region.TwoLetterISORegionName, region.EnglishName);
         }

         this.ViewData.Model = country;
         this.ViewBag.country =
            from pair in countriesDic
            orderby pair.Value
            select new GroupedSelectListItem {
               GroupName = pair.Value.Substring(0, 1),
               Text = pair.Value,
               Value = pair.Key,
               Selected = country != null
                  && country == pair.Key
            };

         return View();
      }

      public ActionResult RadioButtonListFor(DayModel model) {

         this.ViewData.Model = model;
         this.ViewData.SetSelectList(model, m => m.Day, EnumList.Create(typeof(DayOfWeek)));

         return View();
      }
   }
}
